package service

import (
	"database/sql"
	"fmt"
	"os"
	"path/filepath"

	_ "github.com/mattn/go-sqlite3"
)

type MacAddress struct {
	Id         string
	Prefix     string
	VendorName string
	Private    bool
	BlockType  string
	LastUpdate string
}

type SQLiteHelper struct {
	db *sql.DB
}

// create the instance of SQLiteHelper but not initialize the database connection
func NewSQLiteHelper() *SQLiteHelper {
	return &SQLiteHelper{
		db: nil,
	}
}

// check if the directory and file for the database exist
func (helper *SQLiteHelper) CheckBeforeInit() bool {
	userHomeDir, _ := os.UserHomeDir()
	dbFileDir := filepath.Join(userHomeDir, ".macfastlookup")
	dbFileName := "mac_vendors.sqlite3"
	dbFilePath := filepath.Join(dbFileDir, dbFileName)

	if _, err := os.Stat(dbFileDir); os.IsNotExist(err) {
		os.Mkdir(dbFileDir, 0755)
	}

	if _, err := os.Stat(dbFilePath); os.IsNotExist(err) {
		return false
	}

	return true
}

// initialize the database connection
func (helper *SQLiteHelper) InitSQLiteHelper() bool {
	dbFileDir := ".macfastlookup"
	userHomeDir, _ := os.UserHomeDir()
	dbFileName := "mac_vendors.sqlite3"
	dbFilePath := filepath.Join(userHomeDir, dbFileDir, dbFileName)

	connectionString := fmt.Sprintf("file:%s?cache=shared&mode=rwc", dbFilePath)
	db, err := sql.Open("sqlite3", connectionString)

	if err != nil {
		return false
	}

	helper.db = db
	return true
}

func (helper *SQLiteHelper) Close() {
	if helper.db == nil {
		return
	}
	helper.db.Close()
}

func (helper *SQLiteHelper) ReloadDatabase() bool {
	return helper.InitSQLiteHelper()
}

func (helper *SQLiteHelper) QueryMacAddressByPrefix(prefix string) (*MacAddress, error) {
	const tableName = "MacAddresses"
	query := fmt.Sprintf("SELECT * FROM %s WHERE Prefix = ?", tableName)
	row := helper.db.QueryRow(query, prefix)

	var macAddress MacAddress
	err := row.Scan(&macAddress.Id, &macAddress.Prefix, &macAddress.VendorName, &macAddress.Private, &macAddress.BlockType, &macAddress.LastUpdate)
	if err != nil {
		if err == sql.ErrNoRows {
			return nil, nil
		}
		return nil, err
	}

	return &macAddress, nil
}

func (helper *SQLiteHelper) querySingleColumn(tableName, columnName string) (string, error) {
	query := fmt.Sprintf("SELECT %s FROM %s LIMIT 1", columnName, tableName)
	row := helper.db.QueryRow(query)

	var result string
	err := row.Scan(&result)
	if err != nil {
		return "", err
	}

	return result, nil
}

func (helper *SQLiteHelper) QueryVersion() (string, error) {
	return helper.querySingleColumn("Config", "Version")
}
